﻿//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Primitive zur Ansteuerung einer achtstelligen 7-Segment-Anzeige                     //
//  _____________________________________________________                               //
//                                                                                      //
//  Copyright © 2016 Reimar Grasbon (softwork@grasbon.de)                               //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  This software is subjected to the FreeBSD License. Redistribution and use in source //
//  and binary forms, with or without modification, are permitted provided that the     //
//  following conditions are met:                                                       //
//                                                                                      //
//  1. Redistributions of source code must retain the above copyright notice, this list //
//     of conditions and the following disclaimer.                                      //
//                                                                                      //
//  2. Redistributions in binary form must reproduce the above copyright notice, this   //
//     list of conditions and the following disclaimer in the documentation and/or      //
//     other materials provided with the distribution.                                  //
//                                                                                      //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY //
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           //
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  //
//  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,    //
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  //
//  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR  //
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,   //
//  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)  //
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          //
//  POSSIBILITY OF SUCH DAMAGE.                                                         //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

#include <avr/pgmspace.h>
#include "Board.h"
#include "Timer.h"
#include "Display.h"

//////////////////////////////////////////////////////////////////////////////////////////
//  Definitionen  ////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
#define STOP_ANIMATION   255    // Animations-Steuerkode für: Animation beenden
#define CLEAR_DISPLAY    253    // Animations-Steuerkode für: Anzeige löschen

//////////////////////////////////////////////////////////////////////////////////////////
//  Typen  ///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
typedef struct            // Animationsrahmen
{   UInt08  digit;        // · Zeichen: 0 .. 7
    UInt08  segments;     // · Darzustellende Segmente (s. DISPLAY_SetSegment())
    UInt08  duration;     // · Anzeigedauer [10ms]
} AnimFrame;

//////////////////////////////////////////////////////////////////////////////////////////
//  Konstanten  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
// ANIM_TWISTING_DIGITS
static const AnimFrame  anim00[] PROGMEM =
{   // -- -- --
    { 0, 0b0000001, 9 }, { 1, 0b0000001,  9 }, { 2, 0b0000001, 9 }, { 2, 0b0000000, 0 },
    { 3, 0b0000001, 9 }, { 4, 0b0000001,  9 }, { 5, 0b0000001, 9 }, { 5, 0b0000000, 0 },
    { 6, 0b0000001, 9 }, { 7, 0b0000001, 50 },
    // Drehung
    { 0, 0b0100000, 0 }, { 1, 0b0000100,  0 }, { 3, 0b0100000, 0 }, { 4, 0b0000100, 0 },
    { 6, 0b0100000, 0 }, { 7, 0b0000100,  9 }, { 0, 0b0010000, 0 }, { 1, 0b0000010, 0 },
    { 3, 0b0010000, 0 }, { 4, 0b0000010,  0 }, { 6, 0b0010000, 0 }, { 7, 0b0000010, 9 },
    { 0, 0b0000001, 0 }, { 1, 0b0000001,  0 }, { 3, 0b0000001, 0 }, { 4, 0b0000001, 0 },
    { 6, 0b0000001, 0 }, { 7, 0b0000001,  9 }, { 0, 0b0100000, 0 }, { 1, 0b0000100, 0 },
    { 3, 0b0100000, 0 }, { 4, 0b0000100,  0 }, { 6, 0b0100000, 0 }, { 7, 0b0000100, 9 },
    { 0, 0b0010000, 0 }, { 1, 0b0000010,  0 }, { 3, 0b0010000, 0 }, { 4, 0b0000010, 0 },
    { 6, 0b0010000, 0 }, { 7, 0b0000010,  9 }, { 0, 0b0000001, 0 }, { 1, 0b0000001, 0 },
    { 3, 0b0000001, 0 }, { 4, 0b0000001,  0 }, { 6, 0b0000001, 0 }, { 7, 0b0000001, 9 },
    { 0, 0b0100000, 0 }, { 1, 0b0000100,  0 }, { 3, 0b0100000, 0 }, { 4, 0b0000100, 0 },
    { 6, 0b0100000, 0 }, { 7, 0b0000100,  9 }, { 0, 0b0010000, 0 }, { 1, 0b0000010, 0 },
    { 3, 0b0010000, 0 }, { 4, 0b0000010,  0 }, { 6, 0b0010000, 0 }, { 7, 0b0000010, 9 },
    { 0, 0b0000001, 0 }, { 1, 0b0000001,  0 }, { 3, 0b0000001, 0 }, { 4, 0b0000001, 0 },
    { 6, 0b0000001, 0 }, { 7, 0b0000001,90 },
    { STOP_ANIMATION  }
};
// ANIM_ALARM_ENABLE
static const AnimFrame  anim01[] PROGMEM =
{   {  CLEAR_DISPLAY  }, { 0, 0b1110111, 0 }, { 1, 0b0001110,   0 },
    { 4, 0b0000000, 0 }, { 5, 0b0000000, 0 }, { 6, 0b0000000,  20 },
    { 4, 0b1001111, 0 }, { 5, 0b0000100, 0 }, { 6, 0b0010101,  20 },
    { 4, 0b0000000, 0 }, { 5, 0b0000000, 0 }, { 6, 0b0000000,  20 },
    { 4, 0b1001111, 0 }, { 5, 0b0000100, 0 }, { 6, 0b0010101,  20 },
    { 4, 0b0000000, 0 }, { 5, 0b0000000, 0 }, { 6, 0b0000000,  20 },
    { 4, 0b1001111, 0 }, { 5, 0b0000100, 0 }, { 6, 0b0010101, 200 },
    {  STOP_ANIMATION }
};
// ANIM_ALARM_DISABLE
static const AnimFrame  anim02[] PROGMEM =
{   {  CLEAR_DISPLAY  }, { 0, 0b1110111, 0 }, { 1, 0b0001110,   0 },
    { 4, 0b0000000, 0 }, { 5, 0b0000000, 0 }, { 6, 0b0000000,  20 },
    { 4, 0b1110111, 0 }, { 5, 0b0111110, 0 }, { 6, 0b1011011,  20 },
    { 4, 0b0000000, 0 }, { 5, 0b0000000, 0 }, { 6, 0b0000000,  20 },
    { 4, 0b1110111, 0 }, { 5, 0b0111110, 0 }, { 6, 0b1011011,  20 },
    { 4, 0b0000000, 0 }, { 5, 0b0000000, 0 }, { 6, 0b0000000,  20 },
    { 4, 0b1110111, 0 }, { 5, 0b0111110, 0 }, { 6, 0b1011011, 200 },
    {  STOP_ANIMATION  }
};
// ANIM_ALARM
static const AnimFrame  anim03[] PROGMEM =
{   {  CLEAR_DISPLAY  }, { 0, 0b0000110, 9 }, { 0, 0b0000001,  9 }, { 1, 0b0000001, 9 },
    { 2, 0b0000001, 9 }, { 3, 0b0000001, 9 }, { 4, 0b0000001,  9 }, { 5, 0b0000001, 9 },
    { 5, 0b0000000, 0 }, { 6, 0b1110111, 0 }, { 7, 0b0001110, 25 }, { 0, 0b0000000, 9 },
    { 1, 0b0000000, 9 }, { 2, 0b0000000, 9 }, { 3, 0b0000000,  9 }, { 4, 0b0000000, 9 },
    { 5, 0b0000000, 9 },
    { STOP_ANIMATION  }
};
// ANIM_ALARM_PAUSE
static const AnimFrame  anim04[] PROGMEM =
{   {  CLEAR_DISPLAY  }, { 0, 0b1110111, 0 }, { 1, 0b0001110,   0 },
    { 4, 0b0000000, 0 }, { 5, 0b0000000, 0 }, { 6, 0b0000000,  20 },
    { 4, 0b1110111, 0 }, { 5, 0b0111110, 0 }, { 6, 0b1011011,  20 },
    { 4, 0b0000000, 0 }, { 5, 0b0000000, 0 }, { 6, 0b0000000,  20 },
    { 4, 0b1110111, 0 }, { 5, 0b0111110, 0 }, { 6, 0b1011011,  20 },
    { 4, 0b0000000, 0 }, { 5, 0b0000000, 0 }, { 6, 0b0000000,  20 },
    { 4, 0b1110111, 0 }, { 5, 0b0111110, 0 }, { 6, 0b1011011, 200 },
    {  STOP_ANIMATION }
};
// ANIM_GOTO_SLEEP
static const AnimFrame  anim05[] PROGMEM =
{   { 3, 0b0110000, 0 }, { 4, 0b0000110, 10 }, { 3, 0b0000110, 0 }, { 4, 0b0110000, 10 },
    { 3, 0b0000000, 0 }, { 4, 0b0000000,  0 }, { 2, 0b0110000, 0 }, { 5, 0b0000110, 10 },
    { 2, 0b0000110, 0 }, { 5, 0b0110000, 10 }, { 2, 0b0000000, 0 }, { 5, 0b0000000,  0 },
    { 1, 0b0110000, 0 }, { 6, 0b0000110, 10 }, { 1, 0b0000110, 0 }, { 6, 0b0110000, 10 },
    { 1, 0b0000000, 0 }, { 6, 0b0000000,  0 }, { 0, 0b0110000, 0 }, { 7, 0b0000110, 10 },
    { 0, 0b0000110, 0 }, { 7, 0b0110000, 10 }, { 0, 0b0000000, 0 }, { 7, 0b0000000,  0 },
    { STOP_ANIMATION  }
};
// ANIM_RADIO_ON
static const AnimFrame  anim06[] PROGMEM =
{   {  CLEAR_DISPLAY  },
    { 0, 0b0000001, 5 }, { 1, 0b0000001, 5 }, { 2, 0b0000001, 5 }, { 3, 0b0000001, 5 },
    { 3, 0b0110001, 5 },
    { 5, 0b1001111, 0 }, { 6, 0b0000100, 0 }, { 7, 0b0010101, 20 },
    { 5, 0b0000000, 0 }, { 6, 0b0000000, 0 }, { 7, 0b0000000, 20 },
    { 5, 0b1001111, 0 }, { 6, 0b0000100, 0 }, { 7, 0b0010101, 20 },
    { 5, 0b0000000, 0 }, { 6, 0b0000000, 0 }, { 7, 0b0000000, 20 },
    { 5, 0b1001111, 0 }, { 6, 0b0000100, 0 }, { 7, 0b0010101, 80 },
    { 0, 0b0000000, 5 }, { 1, 0b0000000, 5 }, { 2, 0b0000000,  5 }, { 3, 0b0000000, 5 },
    { STOP_ANIMATION  }
};
// ANIM_RADIO_OFF
static const AnimFrame  anim07[] PROGMEM =
{   {  CLEAR_DISPLAY  },
    { 0, 0b0000001, 5 }, { 1, 0b0000001, 5 }, { 2, 0b0000001,  5 }, { 3, 0b0000001, 5 },
    { 3, 0b0110001, 5 },
    { 5, 0b1110111, 0 }, { 6, 0b0111110, 0 }, { 7, 0b1011011, 20 },
    { 5, 0b0000000, 0 }, { 6, 0b0000000, 0 }, { 7, 0b0000000, 20 },
    { 5, 0b1110111, 0 }, { 6, 0b0111110, 0 }, { 7, 0b1011011, 20 },
    { 5, 0b0000000, 0 }, { 6, 0b0000000, 0 }, { 7, 0b0000000, 20 },
    { 5, 0b1110111, 0 }, { 6, 0b0111110, 0 }, { 7, 0b1011011, 80 },
    { 0, 0b0000000, 5 }, { 1, 0b0000000, 5 }, { 2, 0b0000000,  5 }, { 3, 0b0000000, 5 },
    { STOP_ANIMATION  }
};
// ANIM_MENU_OPEN
static const AnimFrame  anim08[] PROGMEM =
{   {  CLEAR_DISPLAY  },
    { 0, 0b0000010, 0 }, { 7, 0b0010000, 10 }, { 0, 0b0100000, 0 }, { 7, 0b0000100, 5 },
    { 0, 0b0000000, 0 }, { 7, 0b0000000,  0 }, { 1, 0b0000010, 0 }, { 6, 0b0010000, 2 },
    { 1, 0b0100000, 0 }, { 6, 0b0000100,  1 }, { 1, 0b0000000, 0 }, { 6, 0b0000000, 0 },
    { 2, 0b0000010, 0 }, { 5, 0b0010000,  1 }, { 2, 0b0100000, 0 }, { 5, 0b0000100, 1 },
    { 2, 0b0000000, 0 }, { 5, 0b0000000,  0 }, { 3, 0b0000010, 0 }, { 4, 0b0010000, 1 },
    { 3, 0b0100000, 0 }, { 4, 0b0000100,  1 }, { 3, 0b0000000, 0 }, { 4, 0b0000000, 0 },
    { 4, 0b0000010, 0 }, { 3, 0b0010000,  1 }, { 4, 0b0100000, 0 }, { 3, 0b0000100, 1 },
    { 4, 0b0000000, 0 }, { 3, 0b0000000,  0 }, { 5, 0b0000010, 0 }, { 2, 0b0010000, 1 },
    { 5, 0b0100000, 0 }, { 2, 0b0000100,  1 }, { 5, 0b0000000, 0 }, { 2, 0b0000000, 0 },
    { 6, 0b0000010, 0 }, { 1, 0b0010000,  1 }, { 6, 0b0100000, 0 }, { 1, 0b0000100, 2 },
    { 6, 0b0000000, 0 }, { 1, 0b0000000,  0 }, { 7, 0b0000010, 0 }, { 0, 0b0010000, 5 },
    { 7, 0b0100000, 0 }, { 0, 0b0000100,  5 }, { 0, 0b0000010, 0 }, { 7, 0b0010000, 5 },
    { 0, 0b0100000, 0 }, { 7, 0b0000100,  5 }, { 0, 0b0000000, 0 }, { 7, 0b0000000, 0 },
    { 1, 0b0000010, 0 }, { 6, 0b0010000,  2 }, { 1, 0b0100000, 0 }, { 6, 0b0000100, 1 },
    { 1, 0b0000000, 0 }, { 6, 0b0000000,  0 }, { 2, 0b0000010, 0 }, { 5, 0b0010000, 1 },
    { 2, 0b0100000, 0 }, { 5, 0b0000100,  1 }, { 2, 0b0000000, 0 }, { 5, 0b0000000, 0 },
    { 3, 0b0000010, 0 }, { 4, 0b0010000,  1 }, { 3, 0b0100000, 0 }, { 4, 0b0000100, 1 },
    { 3, 0b0000000, 0 }, { 4, 0b0000000,  0 }, { 4, 0b0000010, 0 }, { 3, 0b0010000, 1 },
    { 4, 0b0100000, 0 }, { 3, 0b0000100,  1 }, { 4, 0b0000000, 0 }, { 3, 0b0000000, 0 },
    { 5, 0b0000010, 0 }, { 2, 0b0010000,  1 }, { 5, 0b0100000, 0 }, { 2, 0b0000100, 1 },
    { 5, 0b0000000, 0 }, { 2, 0b0000000,  0 }, { 6, 0b0000010, 0 }, { 1, 0b0010000, 1 },
    { 6, 0b0100000, 0 }, { 1, 0b0000100,  2 }, { 6, 0b0000000, 0 }, { 1, 0b0000000, 0 },
    { 7, 0b0000010, 0 }, { 0, 0b0010000,  3 }, { 7, 0b0100000, 0 }, { 0, 0b0000100, 4 },
    { 0, 0b0001000, 0 }, { 1, 0b0000000,  5 },
    { 0, 0b0011000, 0 }, { 1, 0b0001000,  5 },
    { 0, 0b0010001, 0 }, { 1, 0b0011000,  5 },
    { 0, 0b0000011, 0 }, { 1, 0b0110000,  5 },
    { 0, 0b1000010, 0 }, { 1, 0b1100000,  5 },
    { 0, 0b1100010, 0 }, { 1, 0b1000010,  5 },
    { 0, 0b1100011, 0 }, { 1, 0b0000110,  5 },
    { 0, 0b1110111, 0 }, { 1, 0b0001110,  5 },
    { STOP_ANIMATION  }
};
// ANIM_MENU_CLOSE
static const AnimFrame  anim09[] PROGMEM =
{   { 3, 0b0110000, 0 }, { 4, 0b0000110, 10 }, { 3, 0b0000111, 0 }, { 4, 0b0110001, 10 },
    { 3, 0b0000001, 0 }, { 4, 0b0000001,  0 }, { 2, 0b0110000, 0 }, { 5, 0b0000110, 10 },
    { 2, 0b0000110, 0 }, { 5, 0b0110000, 10 }, { 2, 0b0000000, 0 }, { 5, 0b0000000,  0 },
    { 1, 0b0110000, 0 }, { 6, 0b0000110, 10 }, { 1, 0b0000111, 0 }, { 6, 0b0110001, 10 },
    { 1, 0b0000001, 0 }, { 6, 0b0000001,  0 }, { 0, 0b0110000, 0 }, { 7, 0b0000110, 10 },
    { 0, 0b0000111, 0 }, { 7, 0b0110001, 10 }, { 0, 0b0000001, 0 }, { 7, 0b0000001, 50 },
    { 0, 0b0100000, 0 }, { 1, 0b0000100,  0 }, { 3, 0b0100000, 0 }, { 4, 0b0000100,  0 },
    { 6, 0b0100000, 0 }, { 7, 0b0000100,  9 }, { 0, 0b0010000, 0 }, { 1, 0b0000010,  0 },
    { 3, 0b0010000, 0 }, { 4, 0b0000010,  0 }, { 6, 0b0010000, 0 }, { 7, 0b0000010,  9 },
    { 0, 0b0000001, 0 }, { 1, 0b0000001,  0 }, { 3, 0b0000001, 0 }, { 4, 0b0000001,  0 },
    { 6, 0b0000001, 0 }, { 7, 0b0000001,  9 }, { 0, 0b0100000, 0 }, { 1, 0b0000100,  0 },
    { 3, 0b0100000, 0 }, { 4, 0b0000100,  0 }, { 6, 0b0100000, 0 }, { 7, 0b0000100,  9 },
    { 0, 0b0010000, 0 }, { 1, 0b0000010,  0 }, { 3, 0b0010000, 0 }, { 4, 0b0000010,  0 },
    { 6, 0b0010000, 0 }, { 7, 0b0000010,  9 }, { 0, 0b0000001, 0 }, { 1, 0b0000001,  0 },
    { 3, 0b0000001, 0 }, { 4, 0b0000001,  0 }, { 6, 0b0000001, 0 }, { 7, 0b0000001,  9 },
    { 0, 0b0100000, 0 }, { 1, 0b0000100,  0 }, { 3, 0b0100000, 0 }, { 4, 0b0000100,  0 },
    { 6, 0b0100000, 0 }, { 7, 0b0000100,  9 }, { 0, 0b0010000, 0 }, { 1, 0b0000010,  0 },
    { 3, 0b0010000, 0 }, { 4, 0b0000010,  0 }, { 6, 0b0010000, 0 }, { 7, 0b0000010,  9 },
    { 0, 0b0000001, 0 }, { 1, 0b0000001,  0 }, { 3, 0b0000001, 0 }, { 4, 0b0000001,  0 },
    { 6, 0b0000001, 0 }, { 7, 0b0000001, 80 },
    { STOP_ANIMATION  }
};
// ANIM_SETTINGS_OPEN
static const AnimFrame  anim10[] PROGMEM =
{   {  CLEAR_DISPLAY   },
    { 0, 0b0000110,  3 }, { 0, 0b1111110, 3 },
    { 0, 0b1001110,  0 }, { 1, 0b0000110, 3 }, { 1, 0b1111000, 3 }, { 1, 0b1001000, 0 },
    { 2, 0b0000110,  3 }, { 2, 0b1111000, 3 }, { 2, 0b1001000, 0 }, { 3, 0b0000110, 3 },
    { 3, 0b1111000,  3 }, { 3, 0b1001000, 0 }, { 4, 0b0000110, 3 }, { 4, 0b1111000, 3 },
    { 4, 0b1001000,  0 }, { 5, 0b0000110, 3 }, { 5, 0b1111000, 3 }, { 5, 0b1001000, 0 },
    { 6, 0b0000110,  3 }, { 6, 0b1111000, 3 }, { 6, 0b1001000, 0 }, { 7, 0b0000110, 3 },
    { 7, 0b1111000, 50 },
    {  STOP_ANIMATION  }
};
// ANIM_SETTINGS_CLOSE
static const AnimFrame  anim11[] PROGMEM =
{   { 3, 0b0110000, 0 }, { 4, 0b0000110, 10 }, { 3, 0b0000110, 0 }, { 4, 0b0110000, 10 },
    { 3, 0b0000000, 0 }, { 4, 0b0000000,  0 }, { 2, 0b0110000, 0 }, { 5, 0b0000110, 10 },
    { 2, 0b0000110, 0 }, { 5, 0b0110000, 10 }, { 2, 0b0000000, 0 }, { 5, 0b0000000,  0 },
    { 1, 0b0110000, 0 }, { 6, 0b0000110, 10 }, { 1, 0b0000110, 0 }, { 6, 0b0110000, 10 },
    { 1, 0b0000000, 0 }, { 6, 0b0000000,  0 }, { 0, 0b0110000, 0 }, { 7, 0b0000110, 10 },
    { 0, 0b0000110, 0 }, { 7, 0b0110000, 10 }, { 0, 0b0000000, 0 }, { 7, 0b0000000,  0 },
    { STOP_ANIMATION  }
};

//////////////////////////////////////////////////////////////////////////////////////////
//  Daten  ///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
static bool       enabled      = false;
static SInt08     brightness   = 15;
static Animation  currentAnim  = ANIM_NONE;
static UInt08     animFrame    = 0;
static Timer      animTimer    = TIMER(TIMER_MILLISECONDS, 0);
static UInt08     flashDigits  = 0;
static Timer      flashTimer   = TIMER(TIMER_MILLISECONDS, 300);
static bool       flashVisible = false;
static UInt08     buffer[8]    = { 0 };
static bool       modified     = false;

//////////////////////////////////////////////////////////////////////////////////////////
//  Lokale Funktionen  ///////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////
//  CharacterGenerator(): Der Zeichengenerator erzeugt zu dem in 'c' angegebenen        //
//  --------------------  Zeichen das entsprechende Muster darzustellender Segmente.    //
//  Bezüglich der Segment-Kodierung siehe DISPLAY_SetSegment(). Existiert zu dem an-    //
//  angegebenen Zeichen kein Segmentmuster, wird ein entsprechendes 'Notdef'-Zeichen    //
//  (Tofu) generiert.                                                                   //
//                                                                                      //
//  Eingabe: · c:  Darzustellendes Zeichen.                                             //
//  Ausgabe: Das zum Zeichen passende Segmentmuster.                                    //
//////////////////////////////////////////////////////////////////////////////////////////
static UInt08
CharacterGenerator(char c)
{                     //ABCDEFG
    #define NONE_CHAR 0b1001001

    // Zeichengenerator
    static const UInt08 pattern[] PROGMEM =
    {   0b0000000, NONE_CHAR, NONE_CHAR, NONE_CHAR, NONE_CHAR, NONE_CHAR,  //   . . . . .
        NONE_CHAR, NONE_CHAR, NONE_CHAR, NONE_CHAR, NONE_CHAR, NONE_CHAR,  // . . . . . .
        NONE_CHAR, 0b0000001, NONE_CHAR, NONE_CHAR, 0b1111110, 0b0110000,  // . - . . 0 1
        0b1101101, 0b1111001, 0b0110011, 0b1011011, 0b1011111, 0b1110000,  // 2 3 4 5 6 7
        0b1111111, 0b1110011, NONE_CHAR, NONE_CHAR, NONE_CHAR, NONE_CHAR,  // 8 9 . . . .
        NONE_CHAR, NONE_CHAR, NONE_CHAR, 0b1110111, NONE_CHAR, 0b1001110,  // . . . A . C
        NONE_CHAR, 0b1001111, 0b1000111, NONE_CHAR, 0b0110111, 0b0000110,  // . E F . H I
        0b0111100, NONE_CHAR, 0b0001110, NONE_CHAR, NONE_CHAR, 0b1111110,  // J . L . . O
        0b1100111, NONE_CHAR, NONE_CHAR, 0b1011011, NONE_CHAR, 0b0111110,  // P . . S . U
        NONE_CHAR, NONE_CHAR, NONE_CHAR, NONE_CHAR, NONE_CHAR, NONE_CHAR,  // . . . . . .
        NONE_CHAR, NONE_CHAR, NONE_CHAR, 0b0001000, NONE_CHAR, NONE_CHAR,  // . . . _ . .
        0b0011111, 0b0001101, 0b0111101, NONE_CHAR, NONE_CHAR, NONE_CHAR,  // b c d . . .
        0b0010111, 0b0000100, NONE_CHAR, NONE_CHAR, NONE_CHAR, NONE_CHAR,  // h i . . . .
        0b0010101, 0b0011101, NONE_CHAR, NONE_CHAR, 0b0000101, NONE_CHAR,  // n o . . r .
        0b0001111, 0b0011100                                               // t u
    };
    UInt08 segments = NONE_CHAR;
    if( c >= ' ' && c <= 'u' ) segments = pgm_read_byte(&pattern[c - ' ']);
    return segments;
}

//////////////////////////////////////////////////////////////////////////////////////////
//  Flush(): Überträgt den Inhalt von 'buffer' in die Anzeige. Berücksichtigt wird die  //
//  -------  aktuelle Helligkeit in 'brightness' und der Aktivierungszustand in         //
//  'enabled'. Im Vorlauf werden die Register des MAX7219 Treiberchips initialisiert.   //
//                                                                                      //
//  Eingabe: -                                                                          //
//  Ausgabe: -                                                                          //
//////////////////////////////////////////////////////////////////////////////////////////
static void
Flush(void)
{
    // Initialisierung der MAX7219 Register.
    BOARD_SendDisplay(0x09, 0);          // Decode Mode : Keiner
    BOARD_SendDisplay(0x0A, brightness); // Intensity   : 'brightness'
    BOARD_SendDisplay(0x0B, 7);          // Scan Limit  : 8 Zeichen
    BOARD_SendDisplay(0x0C, 1);          // Shutdown    : Aus
    BOARD_SendDisplay(0x0F, 0);          // Display Test: Aus
    // Segmente der Anzeige schalten
    for( UInt08 digit=0; digit<8; digit++ )
    {   UInt08 segment;
        if( enabled && (!(flashDigits & BV(7-digit)) || flashVisible) )
        {   segment = buffer[digit];
        }
        else
        {   // Segmente abschalten, jedoch Punkte der Segmente 3 und 4 stehen lassen
            if( digit == 3 || digit == 4 ) segment = buffer[digit] & 0x80;
            else                           segment = 0;
        }
        BOARD_SendDisplay(digit+1, segment);
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
// Funktionen  ///////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
void
DISPLAY_Enable(bool enable)
{
    if( enable != enabled )
    {   enabled  = enable;
        modified = true;
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
bool
DISPLAY_IsEnabled(void)
{
    return enabled;
}

//////////////////////////////////////////////////////////////////////////////////////////
//  DISPLAY_SetBrightness(): Bestimmt die Helligkeit der Anzeige. Die Helligkeit ist    //
//  -----------------------  unabhängig vom Aktivierungszustand der Anzeige (DISPLAY_-  //
//  Enable()).                                                                          //
//                                                                                      //
//  Eingabe: · newBrightness: Helligkeit der Anzeige: 0 .. 15 (dunkel .. hell)          //
//  Ausgabe: -                                                                          //
//////////////////////////////////////////////////////////////////////////////////////////
void
DISPLAY_SetBrightness(SInt08 newBrightness)
{
    if( newBrightness != brightness )
    {   if     ( newBrightness <  0 ) brightness =  0;
        else if( newBrightness > 15 ) brightness = 15;
        else                          brightness = newBrightness;
        modified = true;
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
//  DISPLAY_SetBrightness(): Liefert die aktuelle Helligkeit der Anzeige. Die           //
//  -----------------------  Helligkeit ist unabhängig vom Aktivierungszustand der      //
//  Anzeige (DISPLAY_Enable()).                                                         //
//                                                                                      //
//  Eingabe: -                                                                          //
//  Ausgabe: Aktuell eingestellte Helligkeit der Anzeige: 0 .. 15 (dunkel .. hell)      //
//////////////////////////////////////////////////////////////////////////////////////////
SInt08
DISPLAY_Brightness(void)
{
    return brightness;
}

//////////////////////////////////////////////////////////////////////////////////////////
//  DISPLAY_SetSegment(): Setzt Segmente der 'digit'ten Siebensegmentanzeige.           //
//  --------------------                                                                //
//                                                                                      //
//  Zuordnung:                                               A                          //
//                                                         #####                        //
//                                                       ##     ##                      //
//               7   6   5   4   3   2   1   0         F ##     ## B                    //
//             +---+---+---+---+---+---+---+---+         ##  G  ##                      //
//  segemnts:  |   | A | B | C | D | E | F | G |           #####                        //
//             +---+---+---+---+---+---+---+---+         ##     ##                      //
//                                                     E ##     ## C                    //
//                                                       ##     ##                      //
//                                                         #####                        //
//                                                           D                          //
//                                                                                      //
//  Eingabe: · digit:    Betroffene Siebensegmentanzeige: 0 .. 7.                       //
//           · segments: Zu schaltende Segmente gem. o.g. Zuordnung.                    //
//  Ausgabe: -                                                                          //
//////////////////////////////////////////////////////////////////////////////////////////
void
DISPLAY_SetSegment(UInt08 digit, UInt08 segments)
{
    if( segments == (buffer[digit] & 0x7F) ) return;    // Keine Änderung
    buffer[digit] = (buffer[digit] & 0x80) | segments;
    if( enabled ) modified = true;
}

//////////////////////////////////////////////////////////////////////////////////////////
//  SetDot(): Setzt den Punkt der 'digit'ten Siebensegmentanzeige.                      //
//  --------                                                                            //
//                                                                                      //
//  Eingabe: · digit: Betroffene Siebensegmentanzeige: 0 .. 7.                          //
//           · on:    'true'/'false' = Punkt ein/aus.                                   //
//  Ausgabe: -                                                                          //
//////////////////////////////////////////////////////////////////////////////////////////
void
DISPLAY_SetDot(UInt08 digit, bool on)
{
    if( !!(buffer[digit] & 0x80) == !!on ) return;      // Keine Änderung
    if( on ) buffer[digit] |= 0x80;
    else     buffer[digit] &= 0x7F;
    modified = true;
}

//////////////////////////////////////////////////////////////////////////////////////////
void
DISPLAY_Clear(void)
{
    for( UInt08 digit=0; digit<8; digit++ ) DISPLAY_SetSegment(digit, 0);
}

//////////////////////////////////////////////////////////////////////////////////////////
void
DISPLAY_Print(const String* string)
{
    UInt08 i = 0;
    while( i < 8 )
    {   char c = string->buffer[i];
        if( c == '\0' ) break;
        DISPLAY_SetSegment(i, CharacterGenerator(c));
        i++;
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
void
DISPLAY_PrintP(const char* string)
{
    UInt08 i = 0;
    while( i < 8 )
    {   char c = pgm_read_byte(&string[i]);
        if( c == '\0' ) break;
        DISPLAY_SetSegment(i, CharacterGenerator(c));
        i++;
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
void
DISPLAY_StartAnim(Animation animation)
{
    currentAnim = animation;
    modified = true;
    if( animation != ANIM_NONE )
    {   animFrame = 0;
        TIMER_Stop(&animTimer);
        flashVisible = true;
        DISPLAY_Proceed();
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
void
DISPLAY_StopAnim(void)
{
    DISPLAY_StartAnim(ANIM_NONE);
}

//////////////////////////////////////////////////////////////////////////////////////////
Animation
DISPLAY_CurrentAnim(void)
{
    return currentAnim;
}

//////////////////////////////////////////////////////////////////////////////////////////
void
DISPLAY_SetBlinkingDigits(UInt08 digits)
{
    if( flashDigits == digits ) return; // Keine Änderung
    flashDigits  = digits;
    flashVisible = true;
    TIMER_Stop(&flashTimer);
    modified = true;
}

//////////////////////////////////////////////////////////////////////////////////////////
//  System  //////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
void
DISPLAY_Startup(void)
{
    DISPLAY_Enable(true);
}

//////////////////////////////////////////////////////////////////////////////////////////
void
DISPLAY_Proceed(void)
{
    if( DISPLAY_CurrentAnim() != ANIM_NONE )
    {   // Animation fortführen
        if( TIMER_HasExpired(&animTimer) )
        {   do
            {   // Tabelle der Animationen
                static const AnimFrame* const table[] PROGMEM =
                {   anim00, anim01, anim02, anim03, anim04, anim05,
                    anim06, anim07, anim08, anim09, anim10, anim11
                };
                // Adresse der aktuellen Animation
                UInt08 index = (UInt08)currentAnim;
                const AnimFrame* frame = (const AnimFrame*)pgm_read_word(&table[index]);
                AnimFrame anim;
                // Nächsten Animationsrahmen holen
                anim.digit    = pgm_read_byte(&frame[animFrame].digit);
                anim.segments = pgm_read_byte(&frame[animFrame].segments);
                anim.duration = pgm_read_byte(&frame[animFrame].duration);
                // Insofern kein Steuercode, Segment der angegebenen Stelle setzen
                if( anim.digit == STOP_ANIMATION )
                {   DISPLAY_StopAnim();
                    break;
                }
                animFrame++;
                if( anim.digit == CLEAR_DISPLAY )
                {   DISPLAY_Clear();
                }
                else
                {   DISPLAY_SetSegment(anim.digit, anim.segments);
                }
                TIMER_Start(&animTimer, anim.duration*10);
            } while( TIMER_HasExpired(&animTimer) );
        }
    }
    else
    {   // Blinkvorgang fortsetzen
        if( enabled && flashDigits != 0 && TIMER_RestartIfExpired(&flashTimer) )
        {   flashVisible = !flashVisible;
            modified = true;
        }
    }
    // Inhalte an Anzeige übertragen
    if( modified )
    {   modified = false;
        Flush();
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
void
DISPLAY_Shutdown(void)
{
    DISPLAY_Enable(false);
}

//////////////////////////////////////////////////////////////////////////////////////////
