﻿//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Tonsignale                                                                          //
//  _____________________________________________________                               //
//                                                                                      //
//  Copyright © 2016 Reimar Grasbon (softwork@grasbon.de)                               //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  This software is subjected to the FreeBSD License. Redistribution and use in source //
//  and binary forms, with or without modification, are permitted provided that the     //
//  following conditions are met:                                                       //
//                                                                                      //
//  1. Redistributions of source code must retain the above copyright notice, this list //
//     of conditions and the following disclaimer.                                      //
//                                                                                      //
//  2. Redistributions in binary form must reproduce the above copyright notice, this   //
//     list of conditions and the following disclaimer in the documentation and/or      //
//     other materials provided with the distribution.                                  //
//                                                                                      //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY //
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           //
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  //
//  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,    //
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  //
//  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR  //
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,   //
//  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)  //
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          //
//  POSSIBILITY OF SUCH DAMAGE.                                                         //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

#include <avr/pgmspace.h>
#include "Board.h"
#include "Sound.h"

//////////////////////////////////////////////////////////////////////////////////////////
//  Konstanten  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
// SOUND_STARTUP
static const Tone  sound00[] PROGMEM = { { NOTE_C6,   5 },
                                         { NOTE_E6,   5 },
                                         { NOTE_G6,   5 },
                                         { NOTE_C7,  15 }, TONE_SEQ_END
                                       };
// SOUND_TWISTING_DIGITS
static const Tone  sound01[] PROGMEM = { { NOTE_C3,   1 }, { NOTE_XX,  4 },
                                         { NOTE_C3,   1 }, { NOTE_XX,  4 },
                                         { NOTE_C3,   1 }, { NOTE_XX,  4 },
                                         { NOTE_C3,   1 }, { NOTE_XX,  4 },
                                         { NOTE_C3,   1 }, { NOTE_XX,  4 },
                                         { NOTE_C3,   1 }, { NOTE_XX,  4 },
                                         { NOTE_C3,   1 }, { NOTE_XX,  4 },
                                         { NOTE_C3,   1 }, { NOTE_XX,  4 },
                                         { NOTE_C3,   1 }, { NOTE_XX,  4 },
                                         { NOTE_C3,   1 }, { NOTE_XX,  4 },
                                         { NOTE_C3,   1 }, { NOTE_XX,  4 },
                                         { NOTE_C3,   1 }, { NOTE_XX,  4 },
                                         { NOTE_C3,   1 }, { NOTE_XX,  4 },
                                         { NOTE_C3,   1 }, { NOTE_XX,  4 }, TONE_SEQ_END
                                       };
// SOUND_TIME_OBTAINED
static const Tone  sound02[] PROGMEM = { { NOTE_C6,   7 },
                                         { NOTE_XX,  20 },
                                         { NOTE_C8,  40 }, TONE_SEQ_END
                                       };
// SOUND_ENC_TICK
static const Tone  sound03[] PROGMEM = { { NOTE_C8,   1 }, TONE_SEQ_END
                                       };
// SOUND_OPEN_MENU
static const Tone  sound04[] PROGMEM = { { NOTE_C6,   7 },
                                         { NOTE_XX,   1 },
                                         { NOTE_C8,  10 }, TONE_SEQ_END
                                       };
// SOUND_CLOSE_MENU
static const Tone  sound05[] PROGMEM = { { NOTE_C8,   5 },
                                         { NOTE_C6,  20 }, TONE_SEQ_END
                                       };
// SOUND_RADIO_ON
static const Tone  sound06[] PROGMEM = { { NOTE_C5,   5 },
                                         { NOTE_C6,   2 },
                                         { NOTE_C7,  10 }, TONE_SEQ_END
                                       };
// SOUND_RADIO_OFF
static const Tone  sound07[] PROGMEM = { { NOTE_C7,   5 },
                                         { NOTE_C6,  10 }, TONE_SEQ_END
                                       };
// SOUND_ALARM_ENABLE
static const Tone  sound08[] PROGMEM = { { NOTE_C6,  10 },
                                         { NOTE_XX,   2 },
                                         { NOTE_C7,  20 }, TONE_SEQ_END
                                       };
// SOUND_ALARM_DISABLE
static const Tone  sound09[] PROGMEM = { { NOTE_C7,  10 },
                                         { NOTE_XX,   2 },
                                         { NOTE_C6,  20 }, TONE_SEQ_END
                                       };
// SOUND_SOUND_ALARM_START
static const Tone  sound10[] PROGMEM = { { NOTE_C5,  10 },
                                         { NOTE_C6,  20 }, TONE_SEQ_END
                                       };
// SOUND_ALARM_SIGNAL
static const Tone  sound11[] PROGMEM = { { NOTE_C7,  10 },
                                         { NOTE_G6,  10 },
                                         { NOTE_F6,  10 },
                                         { NOTE_G6,  10 },
                                         { NOTE_C7,  10 }, { NOTE_XX,  70 },
                                         { NOTE_C5,  50 }, { NOTE_XX, 200 },
                                         TONE_SEQ_REPEAT
                                       };
// SOUND_SNOOZE_START
static const Tone  sound12[] PROGMEM = { { NOTE_C6,  10 }, TONE_SEQ_END
                                       };
// SOUND_ALARM_REARM
static const Tone  sound13[] PROGMEM = { { NOTE_C8,   2 }, { NOTE_XX,  80 },
                                         { NOTE_C8,   2 }, { NOTE_XX,  80 },
                                         { NOTE_C8,   2 }, { NOTE_XX,  80 },
                                         { NOTE_C8,   2 }, TONE_SEQ_END
                                       };
// SOUND_ALARM_PAUSE
static const Tone  sound14[] PROGMEM = { { NOTE_C6,  10 }, { NOTE_XX, 8 },
                                         { NOTE_G5,   8 }, 
                                         { NOTE_E5,   8 }, 
                                         { NOTE_C5,  15 }, TONE_SEQ_END
                                       };
// SOUND_SETTINGS_OPEN
static const Tone  sound15[] PROGMEM = { { NOTE_C6,  10 }, { NOTE_XX,  10 },
                                         { NOTE_E6,   6 },
                                         { NOTE_G6,   6 },
                                         { NOTE_A6,   6 },
                                         { NOTE_C7,  15 }, TONE_SEQ_END
                                       };
// SOUND_SETTINGS_CLOSE
static const Tone  sound16[] PROGMEM = { { NOTE_C7,  10 },
                                         { NOTE_XX,  20 },
                                         { NOTE_G6,   8 },
                                         { NOTE_XX,   3 },
                                         { NOTE_E6,   8 },
                                         { NOTE_XX,   3 },
                                         { NOTE_C6,  10 }, TONE_SEQ_END
                                       };
// SOUND_CHANGE_VALUE_BEGIN
static const Tone  sound17[] PROGMEM = { { NOTE_C7,  20 }, TONE_SEQ_END
                                       };

// SOUND_CHANGE_VALUE_END
static const Tone  sound18[] PROGMEM = { { NOTE_C5,   5 },
                                         { NOTE_C7,  15 },
                                           TONE_SEQ_END
                                       };

// Tabelle der Tonsignale
static const Tone* const table[] PROGMEM =
{  sound00, sound01, sound02, sound03, sound04, sound05, sound06, sound07, sound08,
   sound09, sound10, sound11, sound12, sound13, sound14, sound15, sound16, sound17,
   sound18
};

//////////////////////////////////////////////////////////////////////////////////////////
// Funktionen ////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
void
SOUND_Start(Sound sound)
{
    if( sound == SOUND_NONE ) BOARD_StopToneSequence();    
    else                      BOARD_StartToneSequence((const Tone*)pgm_read_word(&table[sound]));
}

//////////////////////////////////////////////////////////////////////////////////////////
void
SOUND_Stop(void)
{
    SOUND_Start(SOUND_NONE);
}

//////////////////////////////////////////////////////////////////////////////////////////
