﻿//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Zeichenkette                                                                        //
//  _____________________________________________________                               //
//                                                                                      //
//  Copyright © 2006 Reimar Grasbon (softwork@grasbon.de)                               //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  This software is subjected to the FreeBSD License. Redistribution and use in source //
//  and binary forms, with or without modification, are permitted provided that the     //
//  following conditions are met:                                                       //
//                                                                                      //
//  1. Redistributions of source code must retain the above copyright notice, this list //
//     of conditions and the following disclaimer.                                      //
//                                                                                      //
//  2. Redistributions in binary form must reproduce the above copyright notice, this   //
//     list of conditions and the following disclaimer in the documentation and/or      //
//     other materials provided with the distribution.                                  //
//                                                                                      //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY //
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           //
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  //
//  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,    //
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  //
//  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR  //
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,   //
//  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)  //
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          //
//  POSSIBILITY OF SUCH DAMAGE.                                                         //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Hinweis: Um die Datenstruktur Zeichenkette einsetzen zu können muss in Conf.h die   //
//           Konstante 'CFG_STR_CAPACITY' definiert sein. Wertebereich 2 .. 250.        //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////| RG |//

#include <avr/pgmspace.h>
#include "String.h"

//////////////////////////////////////////////////////////////////////////////////////////
//  Funktionen   /////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
const char*
STRING_Adr(const String* string)
{
    return string->buffer;
}

//////////////////////////////////////////////////////////////////////////////////////////
bool
STRING_IsEmpty(const String* string)
{
    return string->length == 0;
}

//////////////////////////////////////////////////////////////////////////////////////////
void
STRING_Clear(String* string)
{
    string->buffer[0] = '\0';
    string->length    = 0;
}

//////////////////////////////////////////////////////////////////////////////////////////
UInt08
STRING_Len(const String* string)
{
    return string->length;
}

//////////////////////////////////////////////////////////////////////////////////////////
char
STRING_Last(const String* string)
{
    if( string->length == 0 ) return '\0';
    return string->buffer[string->length-1];
}

//////////////////////////////////////////////////////////////////////////////////////////
void
STRING_Chop(String* string)
{
    if( string->length > 0 ) string->length--;
    string->buffer[string->length] = '\0';
}

//////////////////////////////////////////////////////////////////////////////////////////
void
STRING_PrintChar(String* string, char c)
{
    if( string->length < CFG_STR_CAPACITY )
    {   string->buffer[string->length] = c;
        string->length++;
        string->buffer[string->length] = '\0';
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
//  STR_PrintTxt(): Überträgt die angegebene Zeichenkette in den String.                //
//  --------------                                                                      //
//                                                                                      //
//  Eingabe: · string: Adresse der String-Struktur.                                     //
//           · text:   Die auszugebende Zeichenkette (darf nicht NULL sein).            //
//  Ausgabe: -                                                                          //
//////////////////////////////////////////////////////////////////////////////////////////
void
STRING_PrintTxt(String* string, const char* text)
{
    while( *text != '\0' )
    {   STRING_PrintChar(string, *text);
        text++;
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
void
STRING_PrintTxtP(String* string, const char* text)
{
    char  c;

    while( true )
    {   c = pgm_read_byte(text);
        if( c == '\0' ) break;
        STRING_PrintChar(string, c);
        text++;
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
//  STRING_PrintNum(): Schreibt den übergebenen Wert als vorzeichenbehaftete Ganzzahl   //
//  -----------------  in den String. Führende Nullen werden unterdrückt.               //
//                                                                                      //
//      Beispiele:  STRING_PrintSInt(1)        -> "1"                                   //
//                  STRING_PrintSInt(-1)       -> "-1"                                  //
//                  STRING_PrintSInt(+32767)   -> "32767"                               //
//                  STRING_PrintSInt(-32768)   -> "-32768"                              //
//                  STRING_PrintSInt(0123)     -> "123"                                 //
//                                                                                      //
//  Eingabe: · string: Adresse der String-Struktur.                                     //
//           · value:  Der auszugebende Wert: -32768 .. +32767.                         //
//  Ausgabe: -                                                                          //
//////////////////////////////////////////////////////////////////////////////////////////
void
STRING_PrintNum(String* string, SInt16 value)
{
    if( value < 0 )
    {   STRING_PrintChar(string, '-');
        if( value == -32768 )
        {   STRING_PrintTxt(string, "32768");
            return;
        }
        value = -value;
    }
    // Vorzeichenlose 15bit Ganzzahl ausgeben
    bool zero  = false;
    UInt16 div = 10000;
    do
    {   UInt16 digit = (value/div)%10;
        if( zero || digit != 0 )
        {   STRING_PrintChar(string, (char)('0'+(UInt08)digit));
            zero = true;
        }
        div /= 10;
        if( div == 1 ) zero = true;
    } while( div != 0 );
}

//////////////////////////////////////////////////////////////////////////////////////////



